using UnityEngine;
using UnityEditor;
using System.Collections;
using System.IO;


[CustomEditor(typeof(TerrainScript))]
public class TerrainPaint : Editor
{
	// global variables
	static string appTitle = "Terrain painting tools v.1.0";
	bool needToSave = false;

	Texture2D mixMap;
	Material currentMaterial;

	// paint variable
	static float brushSize = 6f;
	static float hardness = 1f;
	static bool isPainting = false;

	static Color currentColor;
	int currentTextureNr = 1;
	int oldCurrentTextureNr = 0;

	void createMixTexture ()
	{
		if (needToSave) {
			string path = EditorUtility.SaveFilePanel ("Save texture as?", "", "mixTexture.png", "png");
			if (path.Length != 0) {
				mixMap = new Texture2D (512, 512);
				Color[] c = new Color[512 * 512];
				for (int i = 0; i < c.Length; i++) {
					c[i] = new Color (0, 0, 0, 0);
				}
				mixMap.SetPixels (c);
				byte[] data = mixMap.EncodeToPNG ();
				File.WriteAllBytes (path, data);
				AssetDatabase.Refresh (ImportAssetOptions.ForceSynchronousImport);
				path = path.Substring (path.IndexOf ("Asset"));
				mixMap = (Texture2D)AssetDatabase.LoadAssetAtPath (path, typeof(Texture2D));
			    TextureImporter textureImporter = AssetImporter.GetAtPath(path) as TextureImporter;
				textureImporter.textureFormat = TextureImporterFormat.ARGB32;
				if (textureImporter.isReadable == false)
				{
				   textureImporter.isReadable = true;
				  AssetDatabase.ImportAsset(path, ImportAssetOptions.ForceUpdate);
				}
				currentMaterial.SetTexture ("_Mask", mixMap);
				needToSave = false;
			}
		} else {
			if (mixMap != null) {
				string path = AssetDatabase.GetAssetPath (mixMap);
				if (path.Length > 0) {
					byte[] textureBytes = mixMap.EncodeToPNG ();
					File.WriteAllBytes (path, textureBytes);
				}
				AssetDatabase.Refresh ();
			}
		}
	}
	void clearMap ()
	{
		string path = AssetDatabase.GetAssetPath (mixMap);
		Color[] c = new Color[512 * 512];
		for (int i = 0; i < c.Length; i++) {
			c[i] = new Color (0, 0, 0, 0);
		}
		mixMap.SetPixels (c);
		byte[] textureBytes = mixMap.EncodeToPNG ();
		File.WriteAllBytes (path, textureBytes);
		AssetDatabase.Refresh ();
		
	}

	void paintOnTexture ()
	{
		Event e = Event.current;
		RaycastHit hit;
		Ray ray = HandleUtility.GUIPointToWorldRay (e.mousePosition);
		if (Physics.Raycast (ray, out hit)) {
			Vector2 pixelUV = hit.textureCoord;
			pixelUV.x *= mixMap.width;
			pixelUV.y *= mixMap.height;
			Vector2 start = new Vector2 (Mathf.Clamp (pixelUV.x - brushSize, 0, mixMap.width), Mathf.Clamp (pixelUV.y - brushSize, 0, mixMap.height));
			//float width = brushSize * 2;
			Vector2 end = new Vector2 (Mathf.Clamp (pixelUV.x + brushSize, 0, mixMap.width), Mathf.Clamp (pixelUV.y + brushSize, 0, mixMap.height));
			int widthX = Mathf.RoundToInt (end.x - start.x);
			int widthY = Mathf.RoundToInt (end.y - start.y);
			//float sqrRad = brushSize * brushSize;
			float sqrRad2 = (brushSize + 1) * (brushSize + 1);
			Color[] pixels = mixMap.GetPixels ((int)start.x, (int)start.y, (int)widthX, (int)widthY);
			for (int x = 0; x < widthX; x++) {
				for (int y = 0; y < widthY; y++) {
					Vector2 p = new Vector2 (x, y) + start;
					Vector2 center = p + new Vector2 (0.5f, 0.5f);
					float dist = (center - pixelUV).sqrMagnitude;
					if (dist > sqrRad2) {
						continue;
					}
					Vector2[] samples = Sample (p);
					
					Color c = new Color (0, 0, 0, 0);
					for (int i = 0; i < samples.Length; i++) {
						dist = GaussFalloff (Vector2.Distance (samples[i], pixelUV), brushSize) * hardness;
						if (dist > 0) {
							c += Color.Lerp (pixels[y * (int)widthX + x], currentColor, dist);
						} else {
							c += pixels[y * (int)widthX + x];
						}
					}
					c /= samples.Length;
					pixels[y * (int)widthX + x] = c;
				}
			}
			
			mixMap.SetPixels ((int)start.x, (int)start.y, widthX, widthY, pixels, 0);
			mixMap.Apply ();
			EditorUtility.SetDirty (mixMap);
		}
		
	}

	void setColor ()
	{
		if (oldCurrentTextureNr != currentTextureNr) {
			if (currentTextureNr == 0)
				currentColor = new Color (1, 0, 0, 0);
			if (currentTextureNr == 1)
				currentColor = new Color (0, 1, 0, 0);
			if (currentTextureNr == 2)
				currentColor = new Color (0, 0, 1, 0);
			if (currentTextureNr == 3)
				currentColor = new Color (0, 0, 0, 1);
			oldCurrentTextureNr = currentTextureNr;
			
		}
	}

	//helper functions
	static Vector2[] Sample (Vector2 p)
	{
		
		Vector2[] samples = new Vector2[4];
		samples[0] = p + new Vector2 (0.25f, 0.5f);
		samples[1] = p + new Vector2 (0.75f, 0.5f);
		samples[2] = p + new Vector2 (0.5f, 0.25f);
		samples[3] = p + new Vector2 (0.5f, 0.75f);
		return samples;
		
	}
	public static float GaussFalloff (float dist, float inRadius)
	{
		return Mathf.Clamp01 (Mathf.Pow (360.0f, -Mathf.Pow (dist / inRadius, 2.5f) - 0.01f));
	}

	public override void OnInspectorGUI ()
	{
		// TODO: 
		// check if object has mesh
		// check if object has shader of type ....
		// check perhpas if collider is present, for calculation hit 
		GUILayout.Label (appTitle, EditorStyles.boldLabel);
		GUILayout.Label ("martijn.pixelstudio@gmail.com", EditorStyles.boldLabel);
		GUILayout.Label ("Base Settings", EditorStyles.boldLabel);
		EditorGUIUtility.LookLikeControls (200f, 90f);
		EditorGUILayout.LabelField ("Current selection :", Selection.activeGameObject != null ? Selection.activeGameObject.name : "none");
		EditorGUIUtility.LookLikeControls (100f, 90f);
		
		brushSize = EditorGUILayout.Slider ("brushSize", brushSize, 1f, 100f);
		hardness = EditorGUILayout.Slider ("brushHardness", hardness, 0.01f, 1f);
		
		EditorGUILayout.BeginHorizontal ();
		
		currentTextureNr = (EditorGUILayout.IntSlider ("TextureToPaint", currentTextureNr, 0, 3));
		setColor ();
		EditorGUILayout.EndHorizontal ();
		
		currentMaterial = Selection.activeGameObject.gameObject.renderer.sharedMaterial;
		
		if (currentMaterial.shader.name == "TerrainPaint/FourLayerVL2Pass") {
			if (isPainting) {
				if (GUILayout.Button ("Stop painting")) {
					createMixTexture ();
					isPainting = false;
				}
			} else {
				if (GUILayout.Button ("Start painting (ctrl/cmd + click)")) {
					isPainting = true;
					oldCurrentTextureNr = -1;
					setColor ();
					mixMap = currentMaterial.GetTexture ("_Mask") as Texture2D;
					if (mixMap == null) {
						needToSave = true;
					} else {
						needToSave = false;
					}
				}
				
				mixMap = currentMaterial.GetTexture ("_Mask") as Texture2D;
				if (mixMap != null) {
					if (GUILayout.Button ("Clear mixmap")) {
						clearMap ();
					}
				} else {
					createMixTexture ();
				}
			}
			
		} else {
			GUILayout.Label ("Shader type is not correct! unable to paint");
		}
		
		
	}

	void OnSceneGUI ()
	{
		int ctrlID = GUIUtility.GetControlID (appTitle.GetHashCode (), FocusType.Passive);
		
		if (isPainting) {
			switch (Event.current.type) {
			case EventType.mouseDrag:
				if (Event.current.control || Event.current.command) {
					paintOnTexture ();
					Event.current.Use ();
				}
				break;
			case EventType.layout:
				HandleUtility.AddDefaultControl (ctrlID);
				break;
			}
		}
	}

	[DrawGizmo(GizmoType.SelectedOrChild | GizmoType.NotSelected)]
	static void RenderVisibleVertices (Transform obj, GizmoType gizmoType)
	{
		if ((Selection.activeGameObject != null) && (isPainting)) {
			Event e = Event.current;
			RaycastHit hit;
			Ray ray = HandleUtility.GUIPointToWorldRay (e.mousePosition);
			if (Physics.Raycast (ray, out hit)) {
				Handles.color = Color.cyan;
				Handles.DrawWireDisc (hit.point, hit.normal, brushSize / 100);
			}
		}
		HandleUtility.Repaint ();
	}
	
}
